<?php

/* shelper spell checking function, 
   modified for use with thcnet 1.0.pre9 */

/*
 *
 *	Description:
 *	This is a spell checking function that uses /usr/bin/ispell to check
 *	an form input against a dictionary.  If you don't have ispell, such
 *	as on Windows servers, then i can't help you out with this.  schorry.
 *
 *
 *	Implementation:
 *	Include this function with:  require( "shelper.php" );
 *	Use the function like this:  $outString = spellcheck( $inString );
 *
 *	$inString is the input string ( html tags are allowed ).
 *	$outString is the string returned with changes.
 *
 *	The returned $outString looks exactly like the original $inString
 *	except that any words which did not match against the ispell dictionary
 *	are html tagged in red and bold letters.  You can then write your script
 *	to display $outString to the user so s/he can visually inspect the
 *	returned message and then either re-edit it or submit it to your form 
 *	as good and correct.
 *
 *
 *	Requirements:
 *	1: Your system must have ispell installed and in the path.  If you have 
 *	   it installed but not in the path, edit this script to call the correct
 *	   path for ispell.
 *	2: Your server must be set up to allow the Apache/PHP user, which is
 *	   usually "nobody", to have write access to the directory in which this 
 *	   script runs.  This script creates and deletes two files per call, so 
 *	   it needs a place for those files.  If you prefer, you may use "/tmp" 
 *	   or another "safe" location.
 *
 *
 *	Caveat Emptor:
 *	This script is not elegant.  It is a shell so that you can make changes
 *	for your specific implementation.  It takes in one string and it returns
 *	another.  I did not use a bunch of variables or any kind of initializing
 *	table because my goal is that this script is quick, easy, and simple.  I
 *	could not find any PHP spell checking scripts so I wrote this one.  It 
 *	is probably not the first or only such script, but it is cool! 
 *
 *	This script does not do any kind of type checking on inputs.  There are
 *	so many possible uses for an input field that your code which uses this
 *	script will need to pre-process any special inputs.  The only special
 *	input which this script allows are html tags, using the ispell command
 *	line option "-h".  By using other command line options, you can create
 *	custom dictionaries for common slang, use international dictionaries, 
 *	and a whole host of other cool things.
 *
 *	This script does not do any safety checking on the input string for the 
 *	system call to ispell.  This script uses file redirection, so if there
 *	are any bugs in ispell which allow a redirected file to spawn another
 *	shell or run remote commands, this script is vulnerable to it!  I don't 
 *	believe any such hacks exist but if they do, I am not responsible for
 *	any possible exploits of your web server through this script.  Grab the
 *	latest version at http://www.gnu.org/software/ispell/ispell.html
 *
 *
 *	License:
 *	This script is licensed under the GNU General Public License.  The terms
 *	and conditions of this license are at http://www.gnu.org/copyleft/gpl.html
 *	and I encourage you to give it a read and license all of your own software
 *	creations under the GPL.  I do.
 *
 *	This script is copyleft 2001 by voltaic@thcnet.net, the author.  You may do
 *	whatever you want with it as outlined in the GPL license.  Happy coding.  I
 *	wrote this script initially as a plug-in for the THCNET messaging system
 *	software located at http://freshmeat.net/projects/thcnetmessagesystem/
 *
 *	You can find all of my PHP scripts at http://freshmeat.net/users/voltaic/
 *
 */

function spellcheck( $inString ) {
	mt_srand( (double) microtime() * 1000000 );
	$rfn = mt_rand();
	$scf = fopen( "./in-$rfn", "w+" );
	fwrite( $scf, $inString );
	fclose( $scf );

	$cmdline = "ispell -h -p /usr/local/apache/.ispell_default -l < in-$rfn > out-$rfn";
	system( $cmdline );
	unlink( "in-$rfn" );

	if( filesize( "out-$rfn" ) > 1 ) {
		$winc = 0;
		$scf = fopen( "out-$rfn", "r" );
		while( !feof( $scf ) ) {
			$badwords[ $winc ] = trim( fgets( $scf, 255 ) );
			$goodwords[ $winc ] = "<font color=\"red\"><b>".$badwords[ $winc ]."</b></font>";
			$winc++;
		}
		fclose( $scf );
		$winc--;
                $notalpha = "[^a-zA-Z>]+";
		for( $ridx = 0; $ridx < $winc; $ridx++ ) {
			$inString =  ereg_replace( "($notalpha)(" . $badwords[ $ridx ] . ")($notalpha)", 
                                                   "\\1" . $goodwords[ $ridx ] . "\\3",
                                                   $inString );
		}
	}

        // remove the font tags within the html tags
        $badfonts = "(<[^\"]+\")"
                  . "([^<]+|.?)"
                  . "(<font color=\"red\"><b>)"
                  . "([^<]+)"
                  . "(</b></font>)"
                  . "([^>]+)";
        $goodhtml = "\\1\\2\\4\\6";
        while( eregi( $badfonts, $inString) ) {
	  	$inString =  eregi_replace( $badfonts, $goodhtml, $inString );
	}

        // add link to dictionary.com for misspelled words
        $unlinked = "(<font color=\"red\"><b>)([^<]+)"
                  . "(</b></font>)";
        $linked = "<a href=\"http://www.dictionary.com/cgi-bin/dict.pl?term=\\2\" target=\"_blank\">\\1\\2\\3</a>";
	$inString =  eregi_replace( $unlinked, $linked, $inString );

	unlink( "out-$rfn" );
	return $inString;
}

?>
